SystemOrganization addCategory: #ARAP!

Morph subclass: #ARAPMorph
	instanceVariableNames: 'shape inputs dragging column solver fill ziggle showHandles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ARAP'!

!ARAPMorph class methodsFor: 'instance creation' stamp: 'yo 8/22/2016 12:05'!
newOn: aShape
	^ self new shape: aShape.
! !

!ARAPMorph class methodsFor: 'examples' stamp: 'yo 9/13/2016 10:56'!
truss
	"
	self truss
	"
	(ARAPMorph newOn: Shape truss) fill: Color red lighter; extent: 800@400; openInWorld! !

!ARAPMorph methodsFor: 'drawing' stamp: 'yo 9/13/2016 11:26'!
drawOn: aCanvas
	super drawOn: aCanvas.
	aCanvas translateBy: self position during: [:c |
		fill ifNotNil: [
			shape triangles do: [:tri | | one two three |
				one := (column at: (tri first - 1) * 2 + 1)@(column at: (tri first - 1) * 2 + 2).
				two := (column at: (tri second - 1) * 2 + 1)@(column at: (tri second - 1) * 2 + 2).
				three := (column at: (tri third - 1) * 2 + 1)@(column at: (tri third - 1) * 2 + 2).
				c drawPolygon: {one. two. three} fillStyle: fill.
			].
		] ifNil: [
			shape edges do: [:pair | | dest orig |
				orig := (column at: (pair first - 1) * 2 + 1)@(column at: (pair first - 1) * 2 + 2).
				dest := (column at: (pair second - 1) * 2 + 1)@(column at: (pair second - 1) * 2 + 2).
				c line: orig to: dest width: 2 color: Color gray.
			].
		].
		showHandles ifTrue: [
			1 to: shape vertices size do: [:i |
				| p |
				p := (column at: (i - 1) * 2 + 1)@(column at: (i - 1) * 2 + 2).
				c fillRectangle: ((p - 8) extent: 16) color: ((shape handles includes: i) ifTrue: [Color magenta] ifFalse: [Color black]).
			]
		]
	].! !

!ARAPMorph methodsFor: 'stepping' stamp: 'yo 9/13/2016 10:19'!
fill: anObject

	fill := anObject.
! !

!ARAPMorph methodsFor: 'events' stamp: 'yo 8/22/2016 13:40'!
handlesMouseDown: evt
	^ true.
! !

!ARAPMorph methodsFor: 'events' stamp: 'yo 8/22/2016 13:40'!
handlesMouseMove: evt
	^ dragging notNil
! !

!ARAPMorph methodsFor: 'events' stamp: 'yo 8/22/2016 13:40'!
handlesMouseUp: evt
	^ dragging notNil
! !

!ARAPMorph methodsFor: 'accessing' stamp: 'yo 9/13/2016 10:27'!
hideHandles: aBoolean
	hideHandles := aBoolean.
! !

!ARAPMorph methodsFor: 'events' stamp: 'yo 8/22/2016 14:10'!
mouseDown: evt

	| local dist |
	local := evt cursorPoint - self position.
	inputs keysAndValuesDo: [:k :v |
		dist := v dist: local.
		(v dist: local) < 8 ifTrue: [
			dragging := k.
			self]].
! !

!ARAPMorph methodsFor: 'events' stamp: 'yo 8/23/2016 07:18'!
mouseMove: evt
	| local |
	dragging ifNotNil: [
		local := evt cursorPoint - self position.
		inputs at: dragging put: local.
		column := shape perform: solver with: inputs.
		self changed
	].
! !

!ARAPMorph methodsFor: 'events' stamp: 'yo 8/22/2016 14:06'!
mouseUp: evt
	super mouseUp: evt.
	dragging := nil.
! !

!ARAPMorph methodsFor: 'initialization' stamp: 'yo 9/13/2016 11:26'!
shape: aShape
	| r |
	shape := aShape.
	inputs := Dictionary new.
	ziggle := Dictionary new.
	r := 3@0.
	shape handles do: [:h |
		inputs at: h put: (shape vertices at: h).
		ziggle at: h put: r.
		r := r rotateBy: 0.3 about: 0@0].
	self color: Color white.
	self extent: shape vertices max + (20@20).
	column := Array streamContents: [:c |
		shape vertices do: [:v |
			c nextPut: v x.
			c nextPut: v y]].
	solver := #solve2:.
	showHandles := true.
	fill := nil.
! !

!ARAPMorph methodsFor: 'accessing' stamp: 'yo 9/13/2016 10:56'!
showHandles: aBoolean
	showHandles := aBoolean.
! !

!ARAPMorph methodsFor: 'stepping' stamp: 'yo 9/12/2016 13:46'!
step
	| s c p |
	s := 0.05 sin.
	c := 0.05 cos.
	ziggle keys do: [:k |
		p := ziggle at: k.
		p := (p x * c - (p y * s))@(p x * s + (p y * c)).
		inputs at: k put: ((inputs at: k) + p).
		ziggle at: k put: p].
	column := shape perform: solver with: inputs.
	self changed
! !

!ARAPMorph methodsFor: 'stepping' stamp: 'yo 9/12/2016 13:48'!
stepTime
	^ 50! !

Morph subclass: #PointEditorMorph
	instanceVariableNames: 'origForm vertices'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ARAP'!

!PointEditorMorph methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 15:04'!
drawOn: aCanvas

	aCanvas drawImage: origForm at: bounds origin.
	1 to: vertices size - 1 do: [:i |
		aCanvas line: (vertices at: i) + bounds origin to: (vertices at: i + 1)  + bounds origin width: 1 color: Color blue].
	vertices do: [:v |
		aCanvas fillOval: ((v + bounds origin extent: 0@0) expandBy: 2@2) color: Color red].
! !

!PointEditorMorph methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 14:02'!
handlesMouseDown: evt

	^ true
! !

!PointEditorMorph methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 14:01'!
initializeFor: aForm
	vertices := OrderedCollection new.
	origForm := aForm.
	self extent: origForm extent.
! !

!PointEditorMorph methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 14:28'!
mouseDown: evt

	| p |
	p := evt position - bounds origin.
	Transcript show: p printString; cr.
	vertices addLast: p.
	self changed.! !

!PointEditorMorph methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 14:20'!
verticesOn: aStream
	vertices do: [:v |
		aStream nextPutAll: v printString; cr]! !

Morph subclass: #ShapeViewer
	instanceVariableNames: 'vertices edges neighbors highlightedEdge'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ARAP'!

!ShapeViewer class methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 15:01'!
vertices: vertices edges: edges

	^ self new vertices: vertices edges: edges
! !

!ShapeViewer class methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 16:50'!
vertices: vertices edges: edges neighbors: neighbors

	^ self new vertices: vertices edges: edges neighbors: neighbors! !

!ShapeViewer methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 17:05'!
drawOn: aCanvas

	| v1 v2 quad p e1 |
	aCanvas fillRectangle: bounds color: Color white.
	edges do: [:e |
		v1 := vertices at: e first.
		v2 := vertices at: e second.
		aCanvas line: v1 + bounds origin to: v2  + bounds origin width: 1 color: Color blue].
	vertices do: [:v |
		aCanvas fillOval: ((v + bounds origin extent: 0@0) expandBy: 2@2) color: Color red].

	(highlightedEdge notNil and: [neighbors notNil]) ifTrue: [
		quad := neighbors at: highlightedEdge.
		v1 := vertices at: quad first.
		v2 := vertices at: quad second.
		aCanvas line: v1 + bounds origin to: v2  + bounds origin width: 3 color: Color blue.
		
		e1 := Point r: 8 degrees: ((v2 - v1) theta + 2.6) radiansToDegrees.
		aCanvas line: v2 + bounds origin to: v2  + bounds origin + e1 width: 3 color: Color blue.
		e1 := Point r: 8 degrees: ((v2 - v1) theta - 2.6) radiansToDegrees.
		aCanvas line: v2 + bounds origin to: v2  + bounds origin + e1 width: 3 color: Color blue.
		
		
		quad third ~= 0 ifTrue: [
			p := vertices at: quad third.
			aCanvas fillOval: ((p + bounds origin extent: 0@0) expandBy: 4@4) color: Color green].
		quad fourth ~= 0 ifTrue: [
			p := vertices at: quad fourth.
			aCanvas fillOval: ((p + bounds origin extent: 0@0) expandBy: 4@4) color: Color magenta]]! !

!ShapeViewer methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 16:11'!
handlesMouseMove: evt
	^ true! !

!ShapeViewer methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 16:44'!
isClose: p to: v1 and: v2
	| v t max min |
	v := v2 - v1.
	v = (0@0) ifTrue: [^ false].
	t := p - v1.
	t = (0@0) ifTrue: [^ false].
	v x = 0 ifTrue: [
		^ t x = 0 and: [(t y / v y asFloat) between: 0.0 and: 1.0]].
	min := v1 x min: v2 x.
	max := v1 x max: v2 x.
	^ (p x between: min and: max) and: [((v y * (t x  / v x asFloat)) -  t y) abs < 12]
! !

!ShapeViewer methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 16:52'!
mouseMove: evt
	| p v1 v2 pair |
	(super containsPoint: evt position) ifFalse: [^ self].
	p := evt position - bounds origin.
	highlightedEdge := nil.
	1 to: edges size do: [:i |
		pair := edges at: i.
		v1 := vertices at: pair first.
		v2 := vertices at: pair second.
		(self isClose: p to: v1 and: v2) ifTrue: [
			highlightedEdge := i]].
	self changed.! !

!ShapeViewer methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 15:34'!
vertices: vs edges: es
	self vertices: vs edges: es neighbors: nil
! !

!ShapeViewer methodsFor: 'as yet unclassified' stamp: 'yo 1/13/2017 15:33'!
vertices: vs edges: es neighbors: ns
	| min |
	vertices := vs.
	edges := es.
	neighbors := ns.

	min := vertices min.
	self extent: ((Rectangle encompassing: vs) extent + (40@40) + min).! !

Object subclass: #Shape
	instanceVariableNames: 'vertices edges triangles neighbors handles edgeDiffs a1'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ARAP'!

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:19'!
square
	^ self new vertices: self squareVertices edges: self squareEdges triangles: self squareTriangles neighbors: self squareEdgeNeighbors handles: self squareHandles.! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:18'!
squareEdgeNeighbors
	^ #((1 2 3 0) (1 3 2 5) (1 5 3 0) (2 3 4 1) (2 4 3 0) (3 4 2 5) (3 5 4 1) (4 5 3 0))

	"^ #((1 2 3 0) (2 3 4 1) (3 1 5 2) (3 4 2 5) (4 2 3 0) (4 5 3 0) (5 3 1 4) (1 5 3 0))"

	"
	^ #((1 2) (2 3) (3 1) (3 4) (4 2) (4 5) (5 3) (1 5)).
	^ #((1 2) (1 3) (1 5) (2 3) (2 4) (3 4) (3 5) (4 5))."
	
"	^ {0@0. 2@0. 1@1. 2@2. 0@2}"! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:19'!
squareEdges
	^ #((1 2) (1 3) (1 5) (2 3) (2 4) (3 4) (3 5) (4 5)).
	"^ #((1 2) (2 3) (3 1) (3 4) (4 2) (4 5) (5 3) (1 5))."
! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:19'!
squareHandles
	^ #(1 3).
! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:19'!
squareTriangles
	^ #((1 2 3) (1 3 4) (3 2 4) (3 4 5))
! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:19'!
squareVertices
	^ {10@10. 210@10. 110@110. 210@210. 10@210}
! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:18'!
star
	^ self new vertices: self starVertices edges: self starEdges triangles: self starTriangles neighbors: self starEdgeNeighbors handles: self starHandles.! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/22/2016 15:39'!
starEdgeNeighbors
	^ #((1 7 6 0) (7 2 8 0) (2 8 7 0) (8 3 9 0) (3 9 8 0) (9 4 10 0) (4 10 9 0) (10 5 6 0) (5 6 10 0) (6 1 7 0) (7 6 11 1) (8 7 11 2) (9 8 11 3) (10 9 11 4) (6 10 11 5) (6 11 7 10) (7 11 8 6) (8 11 9 7) (9 11 10 8) (10 11 6 9))! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/22/2016 15:36'!
starEdges
	^ #((1 7) (7 2) (2 8) (8 3) (3 9) (9 4) (4 10) (10 5) (5 6) (6 1) (7 6) (8 7) (9 8) (10 9) (6 10) (6 11) (7 11) (8 11) (9 11) (10 11))
! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 10:09'!
starHandles
	^ #(1 8 9  10) "#(1 2 3 4 5)"
! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:14'!
starTriangles
	^ #((1 6 7) (2 7 8) (3 8 9) (4 9 10) (5 6 10) (6 7 11) (7 8 11) (8 9 11) (9 10 11) (10 6 11))
! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/22/2016 15:27'!
starVertices
	^  {250.0@150.0. 345.10565@219.0983. 308.77852@330.90169. 191.22147@330.90169. 154.89434@219.0983. 225.31301@216.02128. 274.68698@216.02128. 289.94437@262.34349. 250.0@292.0. 210.05562@262.34349. 250@250}! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:18'!
truss
	^ self new vertices: self trussVertices edges: self trussEdges triangles: self trussTriangles neighbors: self trussEdgeNeighbors handles: self trussHandles.! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 13:37'!
trussEdgeNeighbors
	^ #((1 2 3 0) (2 3 1 5) (3 1 2 4) (2 5 3 0) (5 3 2 4) (3 4 1 5) (1 4 3 0)
		(4 5 6 3) (5 6 4 8) (6 4 5 7) (5 8 6 0) (8 6 5 7) (6 7 4 8) (4 7 6 0)
		(7 8 9 6) (8 9 7 11) (9 7 8 10) (8 11 9 0) (11 9 8 10) (9 10 7 11) (7 10 9 0)
		(10 11 12 9) (11 12 10 14) (12 10 11 13) (11 14 12 0) (14 12 11 13) (12 13 10 14) (10 13 12 0)
		(13 14 15 12) (14 15 13 17) (15 13 14 16) (14 17 15 0) (17 15 14 16) (15 16 13 17) (13 16 15 0)
		(16 17 18 15) (17 18 16 20) (18 16 17 19) (17 20 18 0) (20 18 17 19) (18 19 16 20) (16 19 18 0)
		(19 20 21 18) (20 21 19 0) (21 19 20 0) 
	)! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 13:37'!
trussEdges
	^ #((1 2) (2 3) (3 1) (2 5) (5 3) (3 4) (1 4)
		(4 5) (5 6) (6 4) (5 8) (8 6) (6 7) (4 7)
 		(7 8) (8 9) (9 7) (8 11) (11 9) (9 10) (7 10)
		(10 11) (11 12) (12 10) (11 14) (14 12) (12 13) (10 13)
		(13 14) (14 15) (15 13) (14 17) (17 15) (15 16) (13 16)
		(16 17) (17 18) (18 16) (17 20) (20 18) (18 19) (16 19)
		(19 20) (20 21) (21 19) )

! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 13:46'!
trussHandles
	^ #(1 2 6 12 18 21) "#(1 2 3 4 5)"
! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 14:17'!
trussTriangles
	^ #((1 2 3) (2 3 5) (3 1 4) (3 4 5)
	(4 5 6) (5 6 8) (6 4 7) (6 7 8)
	(7 8 9) (8 9 11) (9 7 10) (9 10 11)
	(10 11 12) (11 12 14) (12 10 13) (12 13 14)
	(13 14 15) (14 15 17) (15 13 16) (15 16 17)
	(16 17 18) (17 18 20) (18 16 19) (18 19 20)
	(19 20 21))! !

!Shape class methodsFor: 'as yet unclassified' stamp: 'yo 8/23/2016 13:37'!
trussVertices
	^  {
	0@0. 0@100. 50@50.
	100@0 . 100@100 . 150@50.
	200@0 . 200@100 . 250@50.
	300@0 . 300@100 . 350@50.
	400@0 . 400@100 . 450@50.
	500@0 . 500@100 . 550@50.
	600@0 . 600@100 . 650@50.
	
} + (20@20)! !

!Shape methodsFor: 'naive' stamp: 'yo 8/19/2016 18:18'!
baseSetupA1: input

	a1 := (1 to: (edges size + handles size)) collect: [:e | FloatArray new: vertices size].
	self baseSetupA1Top: input.
	self baseSetupA1Bottom: input.
! !

!Shape methodsFor: 'naive' stamp: 'yo 8/19/2016 19:20'!
baseSetupA1Bottom: input

	1 to: handles size do: [:h |
		| row |
		row := a1 at: edges size + h.
		row at: (handles at: h) put: 1000.
	].
! !

!Shape methodsFor: 'naive' stamp: 'yo 8/19/2016 18:21'!
baseSetupA1Top: input

	1 to: edges size do: [:e |
		| neighbor vi vj row |
		neighbor := neighbors at: e.
		vi := neighbor at: 1.
		vj := neighbor at: 2.
		row := a1 at: e. "(e - 1) * 2 + 1"

		row at: vi put: -1.
		row at: vj put: 1.
	].
! !

!Shape methodsFor: 'naive' stamp: 'yo 8/22/2016 17:03'!
baseSolve: input
	| b a x y result t ti |
	self baseSetupA1: input.
	a := LapackSGEMatrix rows: a1.
	b :=  LapackSGEMatrix rows: (
		(edgeDiffs collect: [:e | {e x}]),
		(handles collect: [:h |
				{1000 * (input at: h ifAbsent: [vertices at: h]) x}])).
	"t := a transposed * a.
	 := reciprocal.
	result := * a transposed * b."
	t := a transposed * a.
	ti := t reciprocal.
	x := ti * a transposed * b.
	b :=  LapackSGEMatrix rows: (
		(edgeDiffs collect: [:e | {e y}]),
		(handles collect: [:h |
				{1000 * (input at: h ifAbsent: [vertices at: h]) y}])).
	t := a transposed * a.
	ti := t reciprocal.
	y := ti * a transposed * b.
	result := Array streamContents: [:st |
		1 to: vertices size do: [:r |
			st nextPut: (x at: r).
			st nextPut: (y at: r)]].
	^ result.

! !

!Shape methodsFor: 'full' stamp: 'yo 8/22/2016 11:35'!
computeT2: i to: input
	| neighbor vi vj vl vr vid vjd vld vrd g gt vds t |
	neighbor := neighbors at: i.
	vi := (neighbor at: 1).
	vj := (neighbor at: 2)..
	vl := (neighbor at: 3)..
	vr := (neighbor at: 4).
	g := {
		{(vertices at: vi) x. (vertices at: vi) y}.
		{(vertices at: vi) y. ((vertices at: vi) x) negated}.
		{(vertices at: vj) x. (vertices at: vj) y}.
		{(vertices at: vj) y. ((vertices at: vj) x) negated}.
		{(vertices at: vl) x. (vertices at: vl) y}.
		{(vertices at: vl) y. ((vertices at: vl) x) negated}}.
	vid := input at: vi ifAbsent: [vertices at: vi].
	vjd := input at: vj ifAbsent: [vertices at: vj].
	vld := input at: vl ifAbsent: [vertices at: vl].
	vds := {
		{vid x}.
		{vid y}.
		{vjd x}.
		{vjd y}.
		{vld x}.
		{vld y}}.
	vr > 0 ifTrue: [
		g := g, {
			{(vertices at: vr) x. (vertices at: vr) y}.
			{(vertices at: vr) y. ((vertices at: vr) x) negated}}.
		vrd := input at: vr ifAbsent: [vertices at: vr].
		vds := vds, {
			{vrd x}.
			{vrd y}}].

	g :=  LapackSGEMatrix rows: g.
	vds := LapackSGEMatrix rows: vds.
	gt := g transposed.
	t := (gt * g) reciprocal * gt * vds.
	^ ((t at: 1)@(t at: 2))
! !

!Shape methodsFor: 'efficient full' stamp: 'yo 8/23/2016 09:54'!
computeT: i to: input
	| neighbor vi vj vl vr g gt |
	neighbor := neighbors at: i.
	vi := (neighbor at: 1).
	vj := (neighbor at: 2)..
	vl := (neighbor at: 3)..
	vr := (neighbor at: 4).
	g := {
		{(vertices at: vi) x. (vertices at: vi) y}.
		{(vertices at: vi) y. ((vertices at: vi) x) negated}.
		{(vertices at: vj) x. (vertices at: vj) y}.
		{(vertices at: vj) y. ((vertices at: vj) x) negated}.
		{(vertices at: vl) x. (vertices at: vl) y}.
		{(vertices at: vl) y. ((vertices at: vl) x) negated}}.
	vr > 0 ifTrue: [
		g := g, {
			{(vertices at: vr) x. (vertices at: vr) y}.
			{(vertices at: vr) y. ((vertices at: vr) x) negated}}].
	g :=  LapackSGEMatrix rows: g.
	gt := g transposed.
	^  (gt * g) reciprocal * gt.
! !

!Shape methodsFor: 'debugging' stamp: 'yo 8/19/2016 16:22'!
drawRectangle: mat with: input
	| f c |
	self halt.
	f := Form extent: 300@300 depth: 32.
	f fillColor: Color white.
	c := f getCanvas.
	1 to: vertices size do: [:i |
		| p |
		p := (mat at: (i - 1) * 2 + 1)@(mat at: (i - 1) * 2 + 2).
		c fillRectangle: ((p - 8) extent: 16) color: Color black.
	].
	edges do: [:pair | | dest orig |
		orig := (mat at: (pair first - 1) * 2 + 1)@(mat at: (pair first - 1) * 2 + 2).
		dest := (mat at: (pair second - 1) * 2 + 1)@(mat at: (pair second - 1) * 2 + 2).
		c line: orig to: dest width: 2 color: Color gray.
	].
	f displayOn: Display.
! !

!Shape methodsFor: 'accessing' stamp: 'yo 8/22/2016 12:04'!
edges
	^ edges
! !

!Shape methodsFor: 'accessing' stamp: 'yo 8/22/2016 13:52'!
handles
	^ handles
! !

!Shape methodsFor: 'full' stamp: 'yo 8/22/2016 11:02'!
setupA12: input

	a1 := (1 to: 2 * (edges size + handles size)) collect: [:e | FloatArray new: 2 * vertices size].
	self setupA1Top2: input.
	self setupA1Bottom: input.
! !

!Shape methodsFor: 'efficient full' stamp: 'yo 8/23/2016 10:39'!
setupA1: input

	a1 := (1 to: 2 * (edges size + handles size)) collect: [:e | FloatArray new: 2 * vertices size].
	self setupA1Top: input.
	self setupA1Bottom: input.
! !

!Shape methodsFor: 'efficient full' stamp: 'yo 8/20/2016 10:05'!
setupA1Bottom: input

	| edgeSize |
	edgeSize := 2 * edges size.
	1 to: handles size do: [:h |
		| xrow yrow |
		xrow := a1 at: edgeSize + ((h - 1) * 2 + 1).
		yrow := a1 at: edgeSize + ((h - 1) * 2 + 2).

		xrow at: ((handles at: h) - 1) * 2 + 1 put: 1000.
		yrow at: ((handles at: h) - 1) * 2 + 2 put: 1000.
	].
! !

!Shape methodsFor: 'full' stamp: 'yo 8/22/2016 11:35'!
setupA1Top2: input

	1 to: edges size do: [:e |
		| neighbor h vi vj xrow yrow |
		neighbor := neighbors at: e.
		"ed := edgeDiffs at: e."
		h := {
			{-1. 0. 1. 0}.
			{0. -1. 0. 1}}.
		vi := neighbor at: 1.
		vj := neighbor at: 2.
		xrow := a1 at: e * 2 - 1. "(e - 1) * 2 + 1"
		yrow := a1 at: e * 2. "(e - 1) * 2 + 2."

		xrow at: (vi - 1)	* 2 + 1 put: ((h at: 1) at: 1).
		xrow at: (vi - 1) * 2 + 2 put: ((h at: 1) at: 2).
		yrow at: (vi - 1)	* 2 + 1 put: ((h at: 2) at: 1).
		yrow at: (vi - 1) * 2 + 2 put: ((h at: 2) at: 2).
	
		xrow at: (vj - 1) * 2 + 1 put: ((h at: 1) at: 3).
		xrow at: (vj - 1) * 2 + 2 put: ((h at: 1) at: 4)..
		yrow at: (vj - 1) * 2 + 1 put: ((h at: 2) at: 3)..
		yrow at: (vj - 1) * 2 + 2 put: ((h at: 2) at: 4).
	].
! !

!Shape methodsFor: 'efficient full' stamp: 'yo 8/19/2016 16:41'!
setupA1Top: input

	1 to: edges size do: [:e |
		| neighbor h t vi vj vl vr xrow yrow |
		neighbor := neighbors at: e.
		t := self computeT: e to: input.
		h := self step1Term: e with: t.
		vi := neighbor at: 1.
		vj := neighbor at: 2.
		vl := neighbor at: 3.
		vr := neighbor at: 4.
		xrow := a1 at: e * 2 - 1. "(e - 1) * 2 + 1"
		yrow := a1 at: e * 2. "(e - 1) * 2 + 2."

		vr > 0 ifTrue: [
			xrow at: (vi - 1)	* 2 + 1 put: (h at: 1).
			xrow at: (vi - 1) * 2 + 2 put: (h at: 2).
			yrow at: (vi - 1)	* 2 + 1 put: (h at: 9).
			yrow at: (vi - 1) * 2 + 2 put: (h at: 10).
	
			xrow at: (vj - 1) * 2 + 1 put: (h at: 3).
			xrow at: (vj - 1) * 2 + 2 put: (h at: 4).
			yrow at: (vj - 1) * 2 + 1 put: (h at: 11).
			yrow at: (vj - 1) * 2 + 2 put: (h at: 12).

			xrow at: (vl - 1)	* 2 + 1 put: (h at: 5).
			xrow at: (vl - 1) * 2 + 2 put: (h at: 6).
			yrow at: (vl - 1) * 2 + 1 put: (h at: 13).
			yrow at: (vl - 1) * 2 + 2 put: (h at: 14).

			xrow at: (vr - 1)	* 2 + 1 put: (h at: 7).
			xrow at: (vr - 1) * 2 + 2 put: (h at: 8).
			yrow at: (vr - 1) * 2 + 1 put: (h at: 15).
			yrow at: (vr - 1) * 2 + 2 put: (h at: 16)
		] ifFalse: [
			xrow at: (vi - 1)	* 2 + 1 put: (h at: 1).
			xrow at: (vi - 1) * 2 + 2 put: (h at: 2).
			yrow at: (vi - 1)	* 2 + 1 put: (h at: 7).
			yrow at: (vi - 1) * 2 + 2 put: (h at: 8).
	
			xrow at: (vj - 1) * 2 + 1 put: (h at: 3).
			xrow at: (vj - 1) * 2 + 2 put: (h at: 4).
			yrow at: (vj - 1) * 2 + 1 put: (h at: 9).
			yrow at: (vj - 1) * 2 + 2 put: (h at: 10).

			xrow at: (vl - 1)	* 2 + 1 put: (h at: 5).
			xrow at: (vl - 1) * 2 + 2 put: (h at: 6).
			yrow at: (vl - 1) * 2 + 1 put: (h at: 11).
			yrow at: (vl - 1) * 2 + 2 put: (h at: 12).
		].
	].
! !

!Shape methodsFor: 'full' stamp: 'yo 8/23/2016 16:39'!
solve2: input
	| b a result t r ti |
	self setupA12: input.
	a := LapackSGEMatrix rows: a1.
	b :=  LapackSGEMatrix rows: (
		Array streamContents: [:row |
			1 to: edges size do: [:e |
				(e = 2 and: [Sensor shiftPressed]) ifTrue: [self halt].
				t := self computeT2: e to: input.
				r := t dotProduct: t.
				row nextPut: {((t x * (edgeDiffs at: e) x + (t y * (edgeDiffs at: e) y))) / r}.
				row nextPut: {((t y negated * (edgeDiffs at: e) x + (t x * (edgeDiffs at: e) y))) / r}
			].
			handles do: [:h |
				row nextPut: {1000 * (input at: h ifAbsent: [vertices at: h]) x}.
				row nextPut: {1000 * (input at: h ifAbsent: [vertices at: h]) y}]]).
	t := a transposed * a.
	ti := t reciprocal.
	result := ti * a transposed * b.
	"c := LapackLeastSquareProblem matrix: a rhsMatrix: b.
	result := c solution."
	"self drawRectangle: result with: input."
	^ result.
! !

!Shape methodsFor: 'efficient full' stamp: 'yo 8/23/2016 07:20'!
solve: input
	| b a c result t ti |
	self setupA1: input.
	a := LapackSGEMatrix rows: a1.
	b :=  LapackSGEMatrix rows: (
		((1 to: 2 * edges size) collect: [:e | #(0)]),
		(Array streamContents: [:bottomB |
			handles do: [:h |
				bottomB nextPut: {1000 * (input at: h ifAbsent: [vertices at: h]) x}.
				bottomB nextPut: {1000 * (input at: h ifAbsent: [vertices at: h]) y}]])).
	"t := a transposed * a.
	ti := t reciprocal.
	result := ti * a transposed * b."
	c := LapackLeastSquareProblem matrix: a rhsMatrix: b.
	result := c solution.
	^ result.
! !

!Shape methodsFor: 'efficient full' stamp: 'yo 8/19/2016 15:14'!
step1Term: e with: gtgig
	| a neighbor vi vj ek em h vl vr |
	neighbor := neighbors at: e.
	vi := neighbor at: 1.
	vj := neighbor at: 2.
	vl := neighbor at: 3.
	vr := neighbor at: 4.
	vr > 0 ifTrue: [
		a := LapackSGEMatrix rows: {
			{-1. 0. 1. 0. 0. 0. 0. 0}.
			{0. -1. 0. 1. 0. 0. 0. 0}}
	] ifFalse: [
		a := LapackSGEMatrix rows: {
			{-1. 0. 1. 0. 0. 0}.
			{0. -1. 0. 1. 0. 0}}].
		
	ek := edgeDiffs at: e.
	em :=  LapackSGEMatrix rows: {
			{ek x. ek y}.
			{ek y. ek x negated}}.
	h := a - (em * gtgig).
	^ h.
! !

!Shape methodsFor: 'accessing' stamp: 'yo 8/23/2016 14:22'!
triangles
	^ triangles
! !

!Shape methodsFor: 'accessing' stamp: 'yo 8/22/2016 12:04'!
vertices
	^ vertices
! !

!Shape methodsFor: 'initialization' stamp: 'yo 8/19/2016 13:57'!
vertices: vs edges: es neighbors: ns handles: hs
	vertices := vs.
	edges := es.
	neighbors := ns.
	handles := hs.
	edgeDiffs := edges collect: [:pair | (vertices at: (pair at: 2)) - (vertices at: (pair at: 1))].! !

!Shape methodsFor: 'initialization' stamp: 'yo 8/23/2016 14:11'!
vertices: vs edges: es triangles: ts neighbors: ns handles: hs
	vertices := vs.
	edges := es.
	triangles := ts.
	neighbors := ns.
	handles := hs.
	edgeDiffs := edges collect: [:pair | (vertices at: (pair at: 2)) - (vertices at: (pair at: 1))].! !
